@@ -15,13 +15,15 @@ Werthmüller for ``empymod``, ``emg3d``, and the ``SimPEG`` framework
 ``watermark.py`` from https://github.com/rasbt/watermark.
 """
 
-from scooby.report import Report
-from scooby.mysteries import in_ipython, in_ipykernel
+from scooby.report import Report, get_version
+from scooby.knowledge import get_standard_lib_modules, in_ipython, in_ipykernel
+from scooby.tracker import TrackedReport, track_imports, untrack_imports
 
-__all__ = ['Report', 'in_ipython', 'in_ipykernel']
+__all__ = ['Report', 'TrackedReport', 'get_standard_lib_modules', 'in_ipython',
+           'in_ipykernel', 'get_version', 'track_imports', 'untrack_imports']
 
 
 __author__ = 'Dieter Werthmüller & Bane Sullivan'
 __license__ = 'MIT'
 __copyright__ = '2019, Dieter Werthmüller & Bane Sullivan'
-__version__ = '0.3.0'
+__version__ = '0.5.0'
