@@ -1,3 +1,12 @@
+"""
+
+report
+======
+
+The main routine containing the `Report` class.
+
+"""
+
 import importlib
 import multiprocessing
 import platform
@@ -6,11 +15,14 @@ import textwrap
 import time
 from types import ModuleType
 
-from scooby.extras import MKL_INFO, TOTAL_RAM, sort_dictionary
-from scooby.knowledge import get_from_knowledge_base
-from scooby.mysteries import in_ipykernel, in_ipython
+from .knowledge import (MKL_INFO, TOTAL_RAM, VERSION_ATTRIBUTES,
+                        VERSION_METHODS, in_ipykernel, in_ipython)
+
+MODULE_NOT_FOUND = 'Could not import'
+VERSION_NOT_FOUND = 'Version unknown'
 
 
+# Info classes
 class PlatformInfo:
     """Internal helper class to access details about the computer platform."""
 
@@ -67,11 +79,11 @@ class PythonInfo:
         self._sort = sort
 
         # Add packages in the following order:
-        self.add_packages(additional)               # Provided by the user
-        self.add_packages(core)                     # Provided by a module dev
-        self.add_packages(optional, optional=True)  # Optional packages
+        self._add_packages(additional)               # Provided by the user
+        self._add_packages(core)                     # Provided by a module dev
+        self._add_packages(optional, optional=True)  # Optional packages
 
-    def add_packages(self, packages, optional=False):
+    def _add_packages(self, packages, optional=False):
         """Add all packages to list; optional ones only if available."""
 
         # Ensure arguments are a list
@@ -84,52 +96,9 @@ class PythonInfo:
 
         # Loop over packages
         for pckg in pckgs:
-            self.get_version(pckg, optional)
-
-    def get_version(self, pckg, optional):
-        """Get the version of a package by passing the package or it's name"""
-
-        if isinstance(pckg, str):  # Case 1: pckg is a string; import it
-            name = pckg
-            pckg = self._safe_import_by_name(pckg)
-
-            # Return if we cannot load the module and it is an optional one
-            if pckg is None and optional:
-                return
-
-        elif isinstance(pckg, ModuleType):  # Case 2: pckg is module; get name
-
-            # Get the name of the package.
-            try:
-                name = pckg.__name__
-            except AttributeError:
-                name = str(pckg).split("'")[1]
-
-        else:
-            raise TypeError("Cannot fetch version from type "
-                            "({})".format(type(pckg)))
-
-        # Now get the version info from the module
-        if pckg is None:
-            version = 'Could not import'
-        else:
-            version = get_from_knowledge_base(pckg, name=name)
-            if version is None:
-                try:
-                    version = pckg.__version__
-                except AttributeError:
-                    version = 'Version unknown'
-
-        # Add the version to the package reference
-        self._packages[name] = version
-
-    def _safe_import_by_name(self, name, optional=False):
-        """Import module `name`; returns None if it fails."""
-        try:
-            module = importlib.import_module(name)
-        except ImportError:
-            module = None
-        return module
+            name, version = get_version(pckg)
+            if not (version == MODULE_NOT_FOUND and optional):
+                self._packages[name] = version
 
     @property
     def sys_version(self):
@@ -148,12 +117,16 @@ class PythonInfo:
         """Return versions of all packages
         (available and unavailable/unknown)
         """
-        packages = dict(self._packages)
+        pckg_dict = dict(self._packages)
         if self._sort:
-            packages = sort_dictionary(packages)
-        return packages
+            packages = {}
+            for name in sorted(pckg_dict.keys(), key=lambda x: x.lower()):
+                packages[name] = pckg_dict[name]
+            pckg_dict = packages
+        return pckg_dict
 
 
+# The main Report instance
 class Report(PlatformInfo, PythonInfo):
     """Have Scooby report the active Python environment.
 
@@ -174,8 +147,8 @@ class Report(PlatformInfo, PythonInfo):
         Defaults to ['numpy', 'scipy', 'IPython', 'matplotlib', 'scooby']
 
     ncol : int, optional
-        Number of package-columns in html table; only has effect if
-        ``mode='HTML'`` or ``mode='html'``. Defaults to 3.
+        Number of package-columns in html table (no effect in text-version);
+        Defaults to 3.
 
     text_width : int, optional
         The text width for non-HTML display modes
@@ -183,9 +156,12 @@ class Report(PlatformInfo, PythonInfo):
     sort : bool, optional
         Sort the packages when the report is shown
 
+    extra_meta : tuple(str, str)
+        Additional two component pairs of meta information to display
+
     """
     def __init__(self, additional=None, core=None, optional=None, ncol=3,
-                 text_width=80, sort=False,):
+                 text_width=80, sort=False, extra_meta=None,):
 
         # Set default optional packages to investigate
         if optional is None:
@@ -196,6 +172,19 @@ class Report(PlatformInfo, PythonInfo):
         self.ncol = int(ncol)
         self.text_width = int(text_width)
 
+        if extra_meta is not None:
+            if not isinstance(extra_meta, (list, tuple)):
+                raise TypeError("`extra_meta` must be a list/tuple of key-value pairs.")
+            if len(extra_meta) == 2 and isinstance(extra_meta[0], str):
+                extra_meta = [extra_meta]
+            for meta in extra_meta:
+                if not isinstance(meta, (list, tuple)) or len(meta) != 2:
+                    raise TypeError("Each chunk of meta info must have two values.")
+        else:
+            extra_meta = []
+        self._extra_meta = extra_meta
+
+
     def __repr__(self):
         """Plain-text version information."""
 
@@ -219,6 +208,8 @@ class Report(PlatformInfo, PythonInfo):
         if TOTAL_RAM:
             text += '{:>18}'.format(self.total_ram)+' : RAM\n'
         text += '{:>18}'.format(self.python_environment)+' : Environment\n'
+        for meta in self._extra_meta:
+            text += '{:>18}'.format(meta[1])+' : {}\n'.format(meta[0])
 
         # ########## Python details ############
         text += '\n'
@@ -228,11 +219,7 @@ class Report(PlatformInfo, PythonInfo):
         text += '\n'
 
         # Loop over packages
-        if self._sort:
-            packages = sort_dictionary(self._packages)
-        else:
-            packages = self._packages
-        for name, version in packages.items():
+        for name, version in self._packages.items():
             text += '{:>18} : {}\n'.format(version, name)
 
         # ########## MKL details ############
@@ -298,6 +285,8 @@ class Report(PlatformInfo, PythonInfo):
             html, i = cols(html, self.total_ram, 'RAM', self.ncol, i)
         html, i = cols(
                 html, self.python_environment, 'Environment', self.ncol, i)
+        for meta in self._extra_meta:
+            html, i = cols(html, meta[1], meta[0], self.ncol, i)
         # Finish row
         html += "  </tr>\n"
 
@@ -325,3 +314,72 @@ class Report(PlatformInfo, PythonInfo):
         html += "</table>"
 
         return html
+
+
+
+# This functionaliy might also be of interest on its own.
+def get_version(module):
+    """Get the version of `module` by passing the package or it's name.
+
+
+    Parameters
+    ----------
+    module : str or module
+        Name of a module to import or the module itself.
+
+
+    Returns
+    -------
+    name : str
+        Package name
+
+    version : str or None
+        Version of module.
+    """
+
+    # module is (1) a string or (2) a module.
+    # If (1), we have to load it, if (2), we have to get its name.
+    if isinstance(module, str):           # Case 1: module is a string; import
+        name = module  # The name is stored in module in this case.
+
+        # Import module `name`; set to None if it fails.
+        try:
+            module = importlib.import_module(name)
+        except ImportError:
+            module = None
+
+    elif isinstance(module, ModuleType):  # Case 2: module is module; get name
+        name = module.__name__
+
+    else:                                 # If not str nor module raise error
+        raise TypeError("Cannot fetch version from type "
+                        "({})".format(type(module)))
+
+    # Now get the version info from the module
+    if module is None:
+        return name, MODULE_NOT_FOUND
+    else:
+
+        # Try common version names.
+        for v_string in ('__version__', 'version'):
+            try:
+                return name, getattr(module, v_string)
+            except AttributeError:
+                pass
+
+        # Try the VERSION_ATTRIBUTES library
+        try:
+            attr = VERSION_ATTRIBUTES[name]
+            return name, getattr(module, attr)
+        except (KeyError, AttributeError):
+            pass
+
+        # Try the VERSION_METHODS library
+        try:
+            method = VERSION_METHODS[name]
+            return name, method()
+        except (KeyError, ImportError):
+            pass
+
+        # If not found, return VERSION_NOT_FOUND
+        return name, VERSION_NOT_FOUND
